"""An implementation of an Alias that works in the Virtual Host Folder"""


# standard issue modules
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_parent, aq_base
import re
# from OFS.FindSupport import FindSupport
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import ObjectManager
from OFS.Folder import Folder

#non-standard Zope requisite:
from Products.OrderedFolder import OrderedObjectManager, SubObjectTypeSupport

# my own

import IHost, IHostOverride, utils

_marker=utils.marker

#bit map settings
DESTINATION_MASK=7
DESTINATION_SHOWN_HIDDEN_MASK=5
DESTINATION_SHOWN_LAST=7
DESTINATION_SHOWN_FIRST=5
DESTINATION_HIDDEN_LAST=3
DESTINATION_HIDDEN_FIRST=1
# 0            0          0
# hidden/shown first/last off/on
FORCE_ALIAS_MASK=24
FORCE_ACTIVE_ALIAS=24
FORCE_PASSIVE_ALIAS=8
# 0              0      (0 0 0)
# passive/active off/on
INCLUDE_PARENTS_MASK=32
INCLUDE_PARENTS=32
# 0      (0 0) (0 0 0)
# off/on
INCLUDE_HOST_ROOT_MASK=64
INCLUDE_HOST_ROOT=64
# 0      (0) (0 0) (0 0 0)
# off/on
RETAIN_PATH_AFTER_ALIAS_MASK=128
RETAIN_PATH_AFTER_ALIAS=128
# 0      (0) (0) (0 0) (0 0 0)
# off/on

SIMPLE_SET_MASK = DESTINATION_MASK + FORCE_ALIAS_MASK + INCLUDE_PARENTS_MASK + INCLUDE_HOST_ROOT_MASK

IS_ACTIVE_MASK=1+FORCE_ACTIVE_ALIAS # that is, destination is on, forcing active alias
INTERPRETATION_MASK=FORCE_ALIAS_MASK+INCLUDE_HOST_ROOT_MASK+DESTINATION_SHOWN_HIDDEN_MASK+INCLUDE_PARENTS_MASK
INTERPRETATION_DICT={
    FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST+INCLUDE_PARENTS:
    "Alias appears to be an actual folder, with acquisition enabled for the paths both to the destination folder and to the alias parent location",
    #FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST:
    #"Alias is in use, attempting to present alias as an actual folder, with acquisition enabled for the paths both to the destination folder and to the virtual host root location",
    FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+INCLUDE_PARENTS:
    "Alias appears to be an actual folder, acquired by the path to the alias parent location (and not the actual destination path)",
    #FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST:
    #"Alias is in use, attempting to present alias as an actual folder, acquired by the path to the virtual host root location (and not the actual destination path)",
    DESTINATION_SHOWN_FIRST:
    "Alias makes no pretense to be anything other than a temporary redirect (useful for images, etc.), without any connection to the virtual host root or parents",
    INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST:
    "Alias uses standard acquisition to find the destination path so that both the virtual host root and the full path to the destination are available",
    INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST+INCLUDE_PARENTS:
    "Alias uses standard acquisition to find the destination path so that both the path to the alias parents and the full path to the destination are available"
    }

class Alias(OrderedObjectManager.OrderedObjectManager, Folder):
    """Represents one alias, an optional part of a virtual host.  Holds alias
    information, and can act as a dynamically-inserted pseudo-folder in
    traversal."""

    __implements__= IHostOverride.IAliasHostOverrideContainer, \
                            IHostOverride.IAliasHostOverride, \
                            IHostOverride.IHostOverride, \
                            IHostOverride.ISimpleAliasHostOverride
    # last two are redundant

    security = ClassSecurityInfo()
    security.declareObjectPublic()
    __allow_access_to_unprotected_subobjects__ = 1
        
    def possible_meta_types(self):
        return ObjectManager.all_meta_types(self, (IHostOverride.ISimpleAliasHostOverride,))

    meta_type="Virtual Host Alias"
    # icon="misc_/VirtualHostFolder/alias"

    manage_options=(
        (
        {'label':'Configuration', 'action':'manage_configure',
        'help':('VirtualHostFolder','Alias_Configure.stx')},
        {'label':'Contents', 'action':'manage_main',
         'help':('VirtualHostFolder','Alias_Contents.stx')}
        )
        + PropertyManager.manage_options
        + SubObjectTypeSupport.SubObjectTypeSupport.manage_options
        + SimpleItem.manage_options
        # + FindSupport.manage_options
        )

    id=parent=mapping=""
    mappingPath=[]
    includeDestinationPath=forceAlias=includeParents=includeHostRoot=0

    def __init__(self):
        """an init"""
        self.aliasName=""
        self.parent=self.mapping="/"
        self.mappingPath=[]
        self.settingsBitmap=0
        self.setRetainPathAfterAlias(1)
        self.showMoreOptions=0

    ## and now we need UI methods

    security.declareProtected( 'View management screens', 'manage_configure' )
    manage_configure = DTMLFile('dtml/AliasManage_main', globals())

    security.declareProtected( 'Manage properties', 'manage_alias' )
    def manage_alias(self, title="", aliasName="", parent="", mapping="", includeDestinationPath=0, includeParents=0, includeHostRoot=0, forceAlias=0, retainPathAfterAlias=0, simple=0, toggleShowMoreOptions=0, REQUEST=None):
        """The basic editing function that manage_configure calls"""
        messages=[]
        mfunc=messages.append
        if toggleShowMoreOptions:
            self.showMoreOptions = not (self.showMoreOptions)
        else:
            ret=0
            ret+=self.setTitle(title)
            if self.showMoreOptions:
                ret+=self.setIncludeDestinationPath(includeDestinationPath, mfunc)
                ret+=self.setForceAlias(forceAlias, mfunc)
                ret+=self.setIncludeParents(includeParents, mfunc)
                ret+=self.setIncludeHostRoot(includeHostRoot, mfunc)
            else:
                ret+=self.setSimple(simple, mfunc)
            ret+=self.setRetainPathAfterAlias(retainPathAfterAlias, mfunc)
            ret+=self.setMapping(mapping, mfunc)
            ret+=self.setParentsAndAliasName(parent, aliasName, mfunc)
            if not ret: mfunc("No changes made")
        if REQUEST is not None:
            return self.manage_configure(self, REQUEST, messages=messages)
        return messages

    security.declarePublic('getTitle')
    def getTitle(self):
        return self.title

    security.declareProtected( 'Manage properties', 'setTitle' )
    def setTitle(self, title):
        self.title=title
        return 1

    security.declarePublic( 'getIsTopLevel' )
    def getIsTopLevel(self):
        """in normal traversal, determines if this Alias is immediately beneath
        a host object or if it is a child of another alias.  Used by the
        manage_configure DTML function"""
        return IHost.IHost.isImplementedBy(aq_parent(self))

    security.declareProtected( 'Manage properties', 'setSimple' )
    def setSimple(self, setting, mfunc=None): # need to check if we are changing alias name to id, and if so if this is ok
        """sets flags wholesale"""
        currSetting=self.settingsBitmap & SIMPLE_SET_MASK
        if currSetting==setting: return 0
        id=self.getId()
        name=self.getTrueAliasName()
        if ((self.getForceAlias() and 1) != (setting & FORCE_ALIAS_MASK) and 1) \
           and (id != name):
            aliasName=id
            if self.getAliasName()==aliasName: aliasName=name
            if not aq_parent(self).checkParentsAndAliasName(self.getParent(), aliasName, self, mfunc):
                if mfunc: mfunc("(changing the alias behavior to this option would change the effective name to %s; change or remove the conflicting sibling name and try again)" % aliasName)
                return 0
        self.settingsBitmap=self.settingsBitmap-currSetting+setting
        self.setFull()
        return 1
    
    security.declarePublic( 'getSimple' )
    def getSimple(self):
        """gets the current state of the flags that setSimple messes with"""
        return self.settingsBitmap & SIMPLE_SET_MASK
    
    security.declareProtected( 'Manage properties', 'setIncludeDestinationPath' )
    def setIncludeDestinationPath(self, setting, mfunc=None):
        """Sets the include destination path flags (see interface for more)"""
        if mfunc and setting==self.getIncludeDestinationPath(): return 0
        if setting not in (0,DESTINATION_SHOWN_LAST, DESTINATION_SHOWN_FIRST, DESTINATION_HIDDEN_LAST, DESTINATION_HIDDEN_FIRST):
            if mfunc:
                mfunc("Not a valid setting for 'Include Destination Path'")
            return 0
        self.settingsBitmap=(self.settingsBitmap&(~DESTINATION_MASK))+setting
        return 1
    
    security.declarePublic( 'getIncludeDestinationPath' )
    def getIncludeDestinationPath(self):
        """gets the include destination path flags"""
        return self.settingsBitmap&DESTINATION_MASK
    
    security.declareProtected( 'Manage properties', 'setForceAlias' )
    def setForceAlias(self, setting, mfunc=None):
        """alias object can be forced in path in an active or passive
        manner.  Setting is here"""
        if mfunc and setting==self.getForceAlias(): return 0
        if setting not in (0,FORCE_ACTIVE_ALIAS, FORCE_PASSIVE_ALIAS):
            if mfunc:
                mfunc("Not a valid setting for 'forceAlias'")
            return 0
        id=self.getId()
        name=self.getTrueAliasName()
        if ((self.getForceAlias() and 1) != (setting & FORCE_ALIAS_MASK) and 1) \
           and (id != name):
            aliasName=id
            if self.getAliasName()==aliasName: aliasName=name
            if not aq_parent(self).checkParentsAndAliasName(self.getParent(), aliasName, self, mfunc):
                if mfunc: mfunc("(changing the alias behavior to this option would change the effective name to %s; change or remove the conflicting sibling name and try again)" % aliasName)
                return 0
        self.settingsBitmap=(self.settingsBitmap&(~FORCE_ALIAS_MASK))+setting
        self.setFull()
        return 1
    
    security.declarePublic( 'getForceAlias' )
    def getForceAlias(self):
        """get whether the alias object is forced in the path (and the manner
        of the forcing)"""
        return self.settingsBitmap&FORCE_ALIAS_MASK
    
    security.declareProtected( 'Manage properties', 'setIncludeParents' )
    def setIncludeParents(self, setting, mfunc=None):
        """sets whether or not the parents of the alias will actually be
        included in the path"""
        setting=setting and INCLUDE_PARENTS or 0
        if mfunc and setting==self.getIncludeParents(): return 0
        self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_PARENTS_MASK))+setting
        if setting and not self.getIncludeHostRoot():
            self.setIncludeHostRoot(1, mfunc)
            if mfunc: mfunc("Including the alias parents in the acquisition path requires that the host root be included, so that setting has been changed as well")
        return 1
    
    security.declarePublic( 'getIncludeParents' )
    def getIncludeParents(self):
        """gets includeParents setting"""
        return self.settingsBitmap&INCLUDE_PARENTS_MASK
    
    security.declareProtected( 'Manage properties', 'setIncludeHostRoot' )
    def setIncludeHostRoot(self, setting, mfunc=None):
        """sets whether or not the host root will be included in the
        acquisition path"""
        setting=setting and INCLUDE_HOST_ROOT or 0
        if mfunc and setting==self.getIncludeHostRoot(): return 0
        if self.getIncludeParents():
            self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_HOST_ROOT_MASK))+INCLUDE_HOST_ROOT
            return setting==INCLUDE_HOST_ROOT
        self.settingsBitmap=(self.settingsBitmap&(~INCLUDE_HOST_ROOT_MASK))+setting
        return 1
    
    security.declarePublic( 'getIncludeHostRoot' )
    def getIncludeHostRoot(self):
        """gets includeHostRoot setting"""
        return self.settingsBitmap&INCLUDE_HOST_ROOT_MASK

    security.declareProtected( 'Manage properties', 'setRetainPathAfterAlias' )
    def setRetainPathAfterAlias(self, setting, mfunc=None):
        """Sets whether the path after alias should be retained in the
        stack.  If it is not retained, the alias' index_html will
        always be returned for any call down this path.  Any
        boolean-equivalent true/false setting will work.  If mfunc is
        included, it may be called with explanatory messages (usually
        mfunc is an array append)."""
        setting=setting and RETAIN_PATH_AFTER_ALIAS or 0
        if mfunc and setting==self.getRetainPathAfterAlias(): return 0
        self.settingsBitmap=(self.settingsBitmap&(~RETAIN_PATH_AFTER_ALIAS_MASK))+setting
        return 1

    security.declarePublic( 'getRetainPathAfterAlias' )
    def getRetainPathAfterAlias(self):
        """Gets the setting described above, returning either 0 (off)
        or a positive integer (on)."""
        return self.settingsBitmap & RETAIN_PATH_AFTER_ALIAS_MASK

    security.declareProtected( 'Manage properties', 'setParentsAndAliasName' )
    def setParentsAndAliasName(self, parent, aliasName, mfunc=None):
        """parents and alias name are set together here so that the request can
        be checked intelligently"""
        if self.getForceAlias():
            aliasName=self.getAliasName()
        else:
            aliasName=aliasName.strip()
        parent=parent.strip()
        if not len(parent) or not parent.startswith("/"): # can I just take out the len check?
            parent="/"+parent
        parent=utils.cleanPath.match(parent).group(1)
        if mfunc and aliasName==self.getAliasName() and parent==self.getParent(): return 0
        if not self.getForceAlias() and not utils.isAFolderName.match(aliasName):
            if mfunc: mfunc("Alias name %s cannot be used.  Alias names must begin with a letter and be at least one character long." % aliasName)
            return 0
        if not aq_parent(self).checkParentsAndAliasName(parent, aliasName, self, mfunc):
            return 0
        if not self.getForceAlias():
            self.aliasName=aliasName
        self.parent=parent
        self.setFull()
        return 1
    
    security.declareProtected( 'View management screens', 'checkParentsAndAliasName' )
    def checkParentsAndAliasName(self, parent, aliasName, caller, mfunc=None):
        """A helper function for setParentsAndAliasName: a function of this
        name will be called in the parent of the given alias"""
        if parent and parent!="/":
            if mfunc: mfunc("An alias within an alias may not specify any parent: it acts as if it resides within the parent alias folder.")
            return 0
        obj= self.getItemById(aliasName)
        if obj and obj!=caller:
            if mfunc: mfunc("An alias of this name already is defined in the parent alias")
            return 0
        return 1
    
    security.declarePublic( 'getMapping' )
    def getMapping(self):
        """get the destination mapping"""
        return self.mapping
        # parent and mapping should begin with a / but not end with one
        
    security.declarePublic( 'getAliasName' )
    def getAliasName(self):
        """get the alias name"""
        if self.getForceAlias():
            return self.getId()
        return self.aliasName
    
    security.declarePublic( 'getTrueAliasName' )
    def getTrueAliasName(self):
        """get the alias name"""
        return self.aliasName
    
    security.declarePublic( 'getParent' )
    def getParent(self):
        """get the parent"""
        return self.parent
    
    security.declarePrivate( 'setFull' )
    def setFull(self, mfunc=None):
        """set the convenience data fullURL and fullPath"""
        par=self.getParent()
        self.fullURL="%s%s%s" % (par, (not par.endswith("/") and "/" or ""), (self.getAliasName()))
        self.fullPath=self.fullURL.split("/")[1:]
        return 1
    
    security.declarePublic( 'getFullURL' )
    def getFullURL(self):
        """get the convenience data fullURL (parent plus aliasName)"""
        return self.fullURL
    
    security.declarePublic( 'getFullPath' )
    def getFullPath(self):
        """get the convenience data fullPath (list version of fullURL)"""
        return self.fullPath
    
    security.declarePrivate( 'setMapping' )
    def setMapping(self, mapping, mfunc=None):
        """set the destination mapping"""
        if mfunc and mapping==self.getMapping(): return 0
        mapping=mapping.strip()
        if not len(mapping) or not mapping.startswith("/"):
            mapping="/"+mapping
        mapping=utils.cleanPath.match(mapping).group(1)
        if not self._check_path(mapping):
            if mfunc:"mapping (destination) folder not found or not a folder."
            return 0
        self.mapping=mapping
        self.mappingPath=self.mapping.split("/")[1:]
        return 1
    
    security.declarePublic( 'getStatus' )
    def getStatus(self, mfunc=None):
        """get the status of the alias, on(bitmap) or off(0).  The bitmap
        contains the setting data for this alias.  If mfunc is included then it
        is called with an interpretation of the current settings."""
        ret=((self.settingsBitmap & IS_ACTIVE_MASK) in (IS_ACTIVE_MASK, 1, FORCE_ACTIVE_ALIAS, 1+FORCE_PASSIVE_ALIAS)) and self.settingsBitmap # or 0
        if not mfunc:
            return ret
        if not ret:
            mfunc("Alias is not in use.  To activate, force alias to be active or include the destination path somewhere in the acquisition path")
            return ret # that is, 0
        ret=ret&INTERPRETATION_MASK
        if INTERPRETATION_DICT.has_key(ret):
            mfunc(INTERPRETATION_DICT[ret])
        else:
            mfunc("Alias is using an unusual setting.  For more information, see the help link")
        if not self.getRetainPathAfterAlias():
            mfunc("Alias will delete all path information after the alias itself: all URLs that have this full URL (parents plus alias name) as their base will be delivered to the index_html method of the destination folder object")
        return self.settingsBitmap

    security.declarePrivate( '_check_path' ) # currently unnecessary
    def _check_path(self,path,mfunc=None):
        """pass in a path, have it validated as bad (empty, not found, or not
        a folder) (0) or ok (1).  The optional mfunc thing is a trick to let the method return a clear on-off numeric success message, while allowing it also to communicate some human-readable messages should we want them.  Just assign a variable in your local scope to an array, pass it along with the path, and then the next time you look at it, through the magic of passing by reference, it might have a message from this function.  This particular function will currently only give you a message if it encounters a bad path."""
        if not path:
           if mfunc:mfunc("Empty path ignored" )
           return 0
        if path.endswith('/'):
           if len(path) == 1:
              return 1
           else:
              path = path[:-1]
        try:
            leaf_object = self.unrestrictedTraverse(path)
        except:
            if mfunc:mfunc("The path %s was not found, so ignored" % path)
            return 0
        if not getattr(leaf_object.aq_base,'isAnObjectManager', None):
            if mfunc:mfunc( "Paths must end in a container: %s does not." % path )
        return 1



    # so stack needs to have path from final object to just before alias destination, "//hostName/aliasName/childAliasName", "VHF", path from alias to virtual root, "/", "VHF", possibly optional path from virtual root to zope root, optional path from destination folder to zope root (may also precede virtual-to-zope-root path instead)
    # This means host needs to return optional alias name string, optional path from alias to virtual root, hidden path.

    security.declarePublic( 'testActivate' )
    def testActivate(self, host, stack, stackString=""):
        """tries to quickly determine if this alias should be used--only called
        in top-level alias objects."""
        if not self.getStatus(): return 0
        return self.fullURL==stackString or stackString.startswith(self.fullURL+"/")

    security.declarePublic( 'activate' )
    def activate(self, testResponse, stack, VHFid, request, path, addressList, matchedStack=[]):
        """Setting the stack as needed--most of the hard work, other than
        __bobo_traverse__"""
        addressList.append(self.getId())
        aliasPath=self.fullPath[:]
        for onePath in aliasPath:
            matchedStack.append(stack.pop())
            assert matchedStack[-1]==onePath, "error in alias set stack--match reported but not found"
        if not self.getIncludeParents() and stack:
            kids=self.objectValues()
            if kids:
                for child in kids:
                    if child.getId()==stack[-1]:
                        if child.getStatus():
                            return child.activate(stack, VHFid, request, path, addressList, matchedStack)
                        else: break

        # If a child, passive alias can't be allowed (or else parent
        # aliases will be searched for next item)

        vhRootString=""
        addressString="/".join(addressList)
        destPath=self.mappingPath[:]
        # destPath.reverse()

        fa=self.getForceAlias()
        idp=self.getIncludeDestinationPath()
        ip=self.getIncludeParents()
        ihr=self.getIncludeHostRoot()
        rpaa=self.getRetainPathAfterAlias()
        
        sa=stack.append

        if not rpaa:
            while stack:
                stack.pop()
        if fa: # not self.includeDestinationPath or
            sa("//"+addressString)
            sa(VHFid)
            if fa==FORCE_ACTIVE_ALIAS:
                destPath.pop()
        if idp==DESTINATION_SHOWN_LAST:
            for i in range(len(destPath)):
                sa(destPath.pop())
        if ip:
            #matchedStack.reverse()
            #stack.extend(matchedStack)
            matchedStack.pop()
            for i in range(len(matchedStack)):
                sa(matchedStack.pop())
        if idp==DESTINATION_SHOWN_FIRST:
            for i in range(len(destPath)):
                sa(destPath.pop())
        if idp==DESTINATION_HIDDEN_LAST or ihr or ip or idp==DESTINATION_HIDDEN_FIRST:
            sa("/")
            sa(VHFid)
            if idp==DESTINATION_HIDDEN_LAST:
                for i in range(len(destPath)):
                    sa(destPath.pop())
                vhRootString=self.mapping+vhRootString
            if ihr or ip: # include Virtual Root
                list_path = path.split('/')[1:]
                for i in range(len(list_path)):
                    sa(list_path.pop())
                vhRootString=path+vhRootString
            if idp==DESTINATION_HIDDEN_FIRST:
                for i in range(len(destPath)):
                    sa(destPath.pop())
                vhRootString=self.mapping+vhRootString
        # request.set( "VIRTUAL_HOST_ROOT_PATH", vhRootString )
        request.set( "VIRTUAL_HOST_IDS", addressString )
        return 1

    def getTraversalResponse(self):
        """A hook that the VHF uses to give the host override an
        opportunity to provide a traversal object"""
        # One could also send a destination list here, instead, but at
        # the moment I don't see any advantage of it over the
        # DESTINATION_SHOWN_... options, and I don't feel like
        # building it out just because I can.  The feature of
        # traversal that allows you to include lists like this as a
        # return from a __bobo_traverse__ seems better suited to
        # as-needed redirect-type objects that are actually placed in
        # a folder, rather than a global before-traversal path
        # rewriting tool like the VHF...
        return aq_base(self)

    def __bobo_traverse__(self, request, name):
        '''Traversing'''
        # first we need to see if the traversal is within the alias's
        # normal location (within a host, acting like a normal
        # item) or if it is somewhere else (acting as a
        # redirector).  Because we use location as the means of
        # determining behavior, trying to mask an alias with an alias
        # is not gonna work too well.  Doesn't seem like too much of a
        # problem.
        ######################################
        ###############parents=request.PARENTS
        ## The above line causes major security problems here, yet works in the VHF.  Why????

        if name=="": name="index_html"

        isNormal=not self.getStatus()
        if not isNormal:
            parent=aq_parent(self)
            while parent and IHostOverride.IAliasHostOverrideContainer.isImplementedBy(parent):
                if IHost.IHost.isImplementedBy(parent):
                    isNormal=1
                    break
                parent=aq_parent(parent)
        if isNormal:
            return self._getOb(name, None)
        # ok...we're not in Kansas; time to do tricks.
        if hasattr(request, "setAliasName"): # standard HTTP request has this because we added it, but other routines use __bobo_traverse__ too, like TAL
            request.setAliasName(self.getId())
        # 1) see if it is a child alias
        for child in self.objectValues():
            if child.getStatus():
                if child.getAliasName()==name: # was getId but I think this is better, or at least what I originally intended in this design; this means that an alias could mask another one...not ideal...
                    return child.__of__(self)
        # 2) see if it is an active alias
        if self.getForceAlias()==FORCE_ACTIVE_ALIAS:
            mapping=self.mapping # change to use getMapping()
            if not mapping or not mapping.endswith("/"): mapping+="/" # change test to use endswith
            address=mapping+name # retainPathAfterAlias handled in activate, btw
            try:
                obj=self.restrictedTraverse(address)
                dest=self.restrictedTraverse(mapping)
                if hasattr(obj, "aq_base"): # is there an instanceof I can look for?
                    if dest==obj or dest in obj.aq_inner.aq_chain:
                        return aq_base(obj).__of__(self)
                # it is a method or simple value of the destination folder or this alias.
                # deal with method first
                elif hasattr(obj, "im_self"):
                    methodSelf=obj.im_self
                    if hasattr(methodSelf, "aq_parent") and (dest==methodSelf or dest in methodSelf.aq_inner.aq_chain):
                        methodSelf.aq_parent=self
                        return obj
                else: # it is a random property.  Let's try to make sure it is a property of the destination
                    alAttr=getattr(aq_base(dest), name, _marker)
                    if alAttr is obj: return obj
                    # so it was acquired from a parent of the dest, so we may not want it.  Let's try to get the value from a parent of the alias (in its fake location)...
                    alAttr=getattr(self, name, _marker)
                    if alAttr is not _marker: return alAttr
                    # ok well let's assume that returning a value from the dest's original context is better than nothing
                    return obj
            except (LookupError, AttributeError): # LookupError includes IndexError and KeyError
                pass # because we then want to try the last step...
        # 3) see if it can be acquired
        return getattr(self, name, None) # that was o =, not return, before I decided that restricted traverse was handling this security stuff without my help
        #if o is not _marker:
            #securityManager=getSecurityManager()
            #if not securityManager.validate( self, None, name, o):
            #    raise 'Unauthorized', name
        #    return o
        #else: return None

    security.declarePublic( 'getVirtualEquivalentPath' )
    def getVirtualEquivalentPath(self, host, pathFromRoot, URLFromRoot):
        """Given a string of the host and a path (list) and URL
        (string) from the Zope root, tries to return a path from the
        virtual root that will use this override or one of its
        children to reach the same location."""
        map=self.getMapping()
        if map==URLFromRoot:
            return self.getFullPath()
        if self.getRetainPathAfterAlias() and URLFromRoot.startswith(map+"/"): # map+"/"==URLFromRoot[:len(map)+1]:
            return self.getFullPath()+URLFromRoot[len(map)+1:].split("/") # ew, I know
        for child in self.objectValues():
            if child.getStatus():
                childpath=child.getVirtualEquivalentPath(host, pathFromRoot, URLFromRoot)
                if childpath:
                    return self.getFullPath()+childpath
        return None

    security.declarePublic( 'getIncludeDestinationPathOptions' )
    def getIncludeDestinationPathOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include in acquisition"),\
            (DESTINATION_HIDDEN_FIRST,"Hide (before virtual root, if present)"),\
            (DESTINATION_HIDDEN_LAST,"Hide (after virtual root, if present)"),\
            (DESTINATION_SHOWN_FIRST,"Show (before alias parents, if present)"),\
            (DESTINATION_SHOWN_LAST,"Show (after alias parents, if present)")\
            #,\
            # (DESTINATION_INSERTED_DYNAMICALLY,"Insert on-the-fly") # requires passive alias.  This functionality has not been done, and may not be..
            )
    
    security.declarePublic( 'getForceAliasOptions' )
    def getForceAliasOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not force alias in path"),\
            (FORCE_PASSIVE_ALIAS,"Force passive alias in path"),\
            (FORCE_ACTIVE_ALIAS,"Force active alias in path")\
            )
    
    security.declarePublic( 'getIncludeParentsOptions' )
    def getIncludeParentsOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include in aquisition path"),\
            (INCLUDE_PARENTS,"Include in aquisition path")\
            )
    
    security.declarePublic( 'getIncludeVirtualRootOptions' )
    def getIncludeHostRootOptions(self):
        """returns options for UI"""
        return (\
            (0,"Do not include virtual root in aquisition path"),\
            (INCLUDE_HOST_ROOT,"Include virtual root in aquisition path")\
            )
    
    def getRetainPathAfterAliasOptions(self):
        """returns options for UI"""
        return (\
            (0,"Delete path after alias"),\
            (RETAIN_PATH_AFTER_ALIAS,"Include path after alias")\
            )
    
    def getSimpleOptions(self):
        """returns simple UI options"""
        return (
    (0,
    "Alias is not in use"),
    (FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+DESTINATION_HIDDEN_FIRST+INCLUDE_PARENTS,
    "Alias appears to be an actual folder, with acquisition enabled for the paths both to the destination folder and to the alias parent location"),
    (FORCE_ACTIVE_ALIAS+INCLUDE_HOST_ROOT+INCLUDE_PARENTS,
    "Alias appears to be an actual folder, acquired by the path to the alias parent location (and not the actual destination path)"),
    (DESTINATION_SHOWN_FIRST,
    "Alias makes no pretense to be anything other than a temporary redirect (useful for images, etc.), without any connection to the virtual host root or parents"),
    (INCLUDE_HOST_ROOT+DESTINATION_SHOWN_FIRST,
    "Alias uses standard acquisition to find the destination path so that both the virtual host root and the full path to the destination are available"),
    (INCLUDE_HOST_ROOT+DESTINATION_SHOWN_LAST+INCLUDE_PARENTS,
    "Alias uses standard acquisition to find the destination path so that both the path to the alias parents and the full path to the destination are available")
    )

manage_addAliasForm=DTMLFile('dtml/addAlias', globals())

def manage_addAlias(dispatcher, id, REQUEST=None, title="", **ignored):
    """Constructor for Alias"""
    container=dispatcher.Destination()
    obj = Alias()
    obj._setId(str(id))
    if title: obj.setTitle(title)
    container._setObject(id, obj)
    if REQUEST:
        return dispatcher.manage_main(dispatcher,REQUEST, update_menu=1)
    else:
        return 1

constructors = (
  ('manage_addAliasForm', DTMLFile('www/addAlias', globals())),
  ('manage_addAlias', manage_addAlias),
)

InitializeClass(Alias)
