from Interface import Base

class IHost( Base ):
    """A class to represent one virtual host, presumably as held by a Virtual
    Host Folder."""

    def testActivate(host, stack, VHFid, request):
        """Given the full host (e.g. 'test.my.site.com'), the
        request's stack (or reasonable testing facsimile), the id of
        the parent Virtual Host Folder, and the request object (or
        reasonable testing facsimile), either return no match (0 or
        other Boolean false) to inform the VHF that this is not the
        correct Host for the job, or call activate and return a
        success message (1 or other Boolean true)."""

    def activate(res, host, stack, VHFid, request):
        """Given the same arguments as testActivate plus, initially, whatever
        the response from testActivate was (to avoid duplicating work, if
        appropriate), modify the request as necessary and return a success
        message."""

    def getVirtualRootURL():
        """Get the canonical virtual host root URL (slash-separated
        string)--that is, the static one that would be used if it were not
        perhaps overwritten by a HostOverride object such as an alias."""

    def getVirtualRootObject():
        """Get the canonical virtual host root object"""

    def getVirtualURLToObject(obj, REQUEST=None):
        """Pass in an object and get a URL (string) that should reach
        it from within the virtual host; needs the REQUEST"""

    def getVirtualPathToObject(obj, REQUEST=None, raiseRedirect=0):
        """Pass in an object and get a path that should reach
        it from within the virtual host; needs the REQUEST.  raiseRedirect is a
        questionable solution to the fact that we are using this solution as the
        guts for getVirtualURLToObject: when calling getVirtualPathToObject
        directly we don't want to raise a redirect error, just know that there is
        no path available, but when this function is the guts of
        getVirtualURLToObject, the caller *wants* the error raised so that it
        can catch it and return it instead."""

class IModifiableHost( IHost ):
    """setters and getters for a basic modifiable host implementation"""

    def setVirtualRootURL(virtualRoot="/", mfunc=None):
        """sets the static virtual root.  Uses optional message system (mfunc)."""

    def addDomainsToList(domains, mfunc=None):
        """adds domains (a list) to the current list of matching
        domains for this virtual host.  Domains are strictly 1-to-1
        matching--no pattern matching of any sort (see the additional
        regex interface below for that).  Calls optional mfunc with
        any pertinent messages."""

    def delDomainsFromList(domains, mfunc=None):
        """deletes list of domains from current list of matching
        domains for this host.  Calls optional mfunc with any
        pertinent messages."""

    def getDomainList():
        """Returns the current list of domains for this virtual
        host."""

class IModifiableRegexHost ( IModifiableHost ):
    """An extension to the IModifiableHost interface that adds regex
    capability."""

    def getVirtualRootWithRegex(matchObj=None, matchObj2=None):
        """a magic function.  Call it with no arguments, and you will
        get the static path.  Call it with a regular expression match
        object or two (designed for given case vs. lowercase), and it
        will try to return the expanded (filled-in) version of the
        dynamic virtual root template; if the template does not exist
        or the expanded path does not resolve to a valid Zope folder,
        the static path is returned."""

    def setVirtualRootTemplate(tmpl, mfunc=None):
        """sets the virtual root template; supports the optional
        messaging system."""

    def getVirtualRootTemplate():
        """returns the unprocessed vitual root template"""

    def setDomainRE(domainRE, ignorecase=1, mfunc=None):
        """sets the domain regular expression.  Pass in anything that
        evaluates to Boolean true in ignorecase to set it to true.  No
        other flags are available (although they could be added, I
        didn't feel they would be useful).  The messages array is an
        optional human-readable messaging system: feel free to
        ignore."""

    def getDomainRE():
        """returns the current domain regular expression *object*"""

    def getDomainREComponents():
        """returns the current domain regular expression and the
        ignorecase flag setting.  If no current domain regular
        expression is set, returns an empty string and the default
        'on' ignorecase setting."""

class ICMFSkinGetterHost ( IHost ):
    """Methods to give the host an ability to work with the CMF Virtual Host
    Tool as a skin getter"""

    def getCMFAllSkins():
        """If the host basic virtual root is in a CMF, will return a list of
        the CMF skins; if not, returns an empty list.  This is a
        convenience function for use with the properties needed for the
        Virtual Host Tool"""

    def getCMFManagerSkins():
        """get the possible owner choices for skins as set by the manager"""

    def getCMFOwnerSkins():
        """get the possible user choices for a skin, as selected by the
        site owner"""

    def getCMFDefaultSkin():
        """get the current default CMF skin"""

class ICMFSkinSetterHost ( ICMFSkinGetterHost ):
    """Methods to give the host an ability to work with the CMF Virtual Host
    Tool as a skin setter"""

    def setCMFManagerSkins(valSeq, mfunc=None):
        """set possible owner choices acording to manager"""

    def setCMFOwnerSkins(valSeq, mfunc=None):
        """set possible user choices according to the owner"""

    def setCMFDefaultSkin(skinName, mfunc=None):
        """set default CMF skin for this host"""
