"""An implementation of a Redirect that works in the Virtual Host Folder"""


# standard issue modules
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Acquisition import aq_parent, aq_base
from ZPublisher.HTTPRequest import HTTPRequest
import re
from OFS.SimpleItem import SimpleItem

# my own

import IHostOverride, utils


class Redirect(SimpleItem):
    """Represents one redirect, an optional part of a virtual host.  Holds
    redirect information, and calls a redirect within BeforeTraverse if there is a match."""

    __implements__= IHostOverride.IHostOverride, \
                    IHostOverride.ISimpleRedirectHostOverride, \
                    IHostOverride.IRedirectHostOverride

    security = ClassSecurityInfo()
    # security.declareObjectPublic()

    meta_type="Virtual Host Redirect"
    # icon="misc_/VirtualHostFolder/redirect"

    manage_options=(
        (
        {'label':'Configuration', 'action':'manage_configure',
        'help':('VirtualHostFolder','Redirect_Configure.stx')},
        )
        +SimpleItem.manage_options
        )


    def __init__(self):
        """an init"""
        self.regex=None
        self.regexTemplate=""

    ## and now we need UI methods

    security.declareProtected( 'View management screens', 'manage_configure' )
    manage_configure = DTMLFile('dtml/RedirectManage_main', globals())

    security.declareProtected( 'Manage properties', 'manage_alias' )
    def manage_redirect(self, title="", regex="", ignorecase="", template="", REQUEST=None):
        """The basic editing function that manage_configure calls"""
        messages=[]
        mfunc=messages.append
        ret=0
        ret+=self.setTitle(title)
        ret+=self.setRegex(regex, ignorecase, mfunc)
        ret+=self.setRegexTemplate(template, mfunc)
        if not ret: mfunc("No changes made")
        if REQUEST is not None:
            return self.manage_configure(self, REQUEST, messages=messages)
        return messages

    security.declarePublic('getTitle')
    def getTitle(self):
        return self.title

    security.declareProtected( 'Manage properties', 'setTitle' )
    def setTitle(self, title):
        self.title=title
        return 1

    security.declareProtected( 'Manage properties', 'setRegexTemplate' )
    def setRegexTemplate(self, tmpl, mfunc=None):
        """sets the template; supports the optional
        messaging system."""
        if tmpl==self.regexTemplate: return 0
        if not tmpl:
            if mfunc:mfunc("Template deleted.")
            self.regexTemplate=""
            return 1
        self.regexTemplate=tmpl
        if mfunc:mfunc("Template changed")
        return 1

    security.declarePublic( 'getRegexTemplate' )
    def getRegexTemplate(self):
        """returns the unprocessed template"""
        return self.regexTemplate

    security.declareProtected( 'Manage properties', 'setRegex' )
    def setRegex(self, regex, ignorecase=1, mfunc=None):
        """sets the domain regular expression.  Pass in anything that evaluates
        to Boolean true in ignorecase to set it to true.  No other flags are
        available (although they could be added, I didn't feel they would be
        useful)."""
        if (not regex and not self.regex) or (self.regex and regex==self.regex.pattern and (not ignorecase)==(not self.regex.flags)): return 0
        if regex:
            test=0
            try:
                test=re.compile(regex, ignorecase and re.I or 0)
            except (SyntaxError, re.error), e:
                # get_transaction().abort()
                if mfunc:mfunc("The regular expression could not be changed. The most likely error is a syntax error in your regular expression.  Python reports the following: %s" % "; ".join(utils.getErrorText(e)))
                return 0
            if mfunc:mfunc("The regular expression is %s" % (self.regex and "changed" or "added"))
            self.regex=test
        else:
            self.regex=None
            if mfunc:mfunc("The regular expression is deleted")
        return 1

    security.declarePublic( 'getRegex' )
    def getRegex(self):
        """returns the current regular expression *object*"""
        return self.regex

    security.declareProtected( 'Manage properties', 'getRegexComponents' )
    def getRegexComponents(self):
        """returns the current regular expression and the ignorecase
        flag setting.  If no current regular expression is set, returns an empty string and the default 'on' ignorecase setting."""
        return self.regex and (self.regex.pattern, self.regex.flags and 1) or ("", 1)

    security.declarePublic( 'testActivate' )
    def testActivate(self, host, stack, stackString=""):
        """determines if this redirect should be used"""
        return self.regex and self.regex.match(stackString)

    security.declarePublic( 'activate' )
    def activate(self, testResponse, stack=None, VHFid="", request={}, path="", addressList=[], matchedStack=[]):
        """calls redirect"""
        ## if isinstance(request, HTTPRequest):
        ##     return request.RESPONSE.redirect(testResponse.expand(self.regexTemplate))
        raise "Redirect", testResponse.expand(self.regexTemplate)

manage_addRedirectForm=DTMLFile('dtml/addRedirect', globals())

def manage_addRedirect(dispatcher, id, REQUEST=None, title="", **ignored):
    """Constructor for Redirect"""
    container=dispatcher.Destination()
    obj = Redirect()
    obj._setId(str(id))
    if title: obj.setTitle(title)
    container._setObject(id, obj)
    if REQUEST:
        return dispatcher.manage_main(dispatcher,REQUEST, update_menu=1)
    else:
        return 1

constructors = (
  ('manage_addRedirectForm', DTMLFile('www/addAlias', globals())),
  ('manage_addRedirect', manage_addRedirect),
)

InitializeClass(Redirect)
